import logging

import spread_knx as knx

from ..managers import Manager
from . import provdiers


def coroutine(func):
    def inner(*args, **kwargs):
        g = func(*args, **kwargs)
        g.send(None)
        return g
    return inner


class Knx(Manager):
    def __init__(self, p_id, m_id):
        super().__init__(p_id, m_id)
        self._server_id = None
        self._broadcast = None
        self.dump_topic = None
        self.send_topic = None

    def parse(self, data):
        super(Knx, self).parse(data)
        if self.send_interface:
            self.send_interface.set_bus_params(self.bus)

    @property
    def send_interface(self):
        return self._send_interface

    @send_interface.setter
    def send_interface(self, interface):
        self._send_interface = interface

    def on_exit(self):
        self._event.clear()
        self._external_cmd.clear()
        super(Knx, self).on_exit()

    @send_interface.setter
    def send_interface(self, interface):
        if interface:
            interface.set_event_handler(self.event_handler())
            interface.set_external_cmd_handler(self.external_command_handler())
        self._send_interface = interface

    def on_external_command(self, command, response, *args):
        pass

    def on_event(self, event_data: knx.commands.KnxFrame):
        for provider in self.providers.values():
            if isinstance(provider, provdiers.KnxProvider):
                if event_data.address == provider.address.state or event_data.address == provider.address.control:
                    dptu = provider.def_state.empty()
                    dptu.fill(event_data.dptu.frame)
                    provider.on_request_answer(dptu)

    @coroutine
    def external_command_handler(self):
        while True:
            try:
                ext = yield
                try:
                    self.on_external_command(*ext)
                except BaseException as ex:
                    logging.debug(f'EXTERNAL FRAME ERROR[{ext}]')
                    logging.exception(ex)
            except StopIteration:
                break

    @coroutine
    def event_handler(self):
        while True:
            try:
                event = yield
                try:
                    self.on_event(event)
                except BaseException as ex:
                    logging.debug(f'EVENT FRAME ERROR[{event}]')
                    logging.exception(ex)
            except StopIteration:
                break
