import datetime

from astral import sun, Observer
from django.views.generic import TemplateView

from spread_core import mqtt
from spread_core.bam import lom
from spread_core.bam.lom.manager import Lom
from ..project import bam
from ..responses import Success


class LomDimmer(TemplateView):
    template_name = 'lom/dimmer.html'

    def get(self, request, *args, **kwargs):
        provider_id = kwargs.get('id')

        try:
            attributes = bam.models.Manager.objects.filter(type=Lom.__name__)[0].attributes
            observer = Observer(attributes['latitude'], attributes['longitude'], attributes['altitude'])
            now = datetime.datetime.now(datetime.timezone.utc)
            s = sun.sun(observer, date=now.date(), tzinfo=datetime.timezone.utc)

            sunrise = s['sunrise'].hour*3600 + s['sunrise'].minute*60 + s['sunrise'].second
            sunset = s['sunset'].hour*3600 + s['sunset'].minute*60 + s['sunset'].second
        except:
            sunrise = 540 * 60
            sunset = 1080 * 60

        if request.GET:
            return Success(True)
        return self.render_to_response(dict(
            providers=[
                {
                    'name': p.name if p.name else 'id:%s' % p.id,
                    'mc': p.type != lom.LomDimmer.__name__,
                    'state': mqtt.spread.topic.State(
                        mqtt.spread.address.ProviderAddress(
                            bam.project.id, Lom.__name__, p.managerID, p.type, p.id, '')
                    ),
                }
                for p in bam.models.Provider.objects.filter(
                    type__in=[lom.LomDimmer.__name__, lom.LomGroup.__name__]
                )
            ],
            readonly=['Binding', 'DevEUI', 'Time', 'OperTime', 'Latitude', 'Longitude', 'Altitude', 'Temp', 'TempMin',
                      'TempMax', 'Illumination', 'Tilt', 'Flags'],
            broker=bam.broker_auth(),
            profiles=mqtt.local.LomProfiles.empty(bam.project.id),
            sun=dict(
                sunrise=sunrise,
                sunset=sunset
            )
        ))
